/* -*-C-*-
                                                                               
 
  File:         compare.c
  RCS:          $Header: $
  Description:  
  Author:       Erwin Dieterich
  Created:      Mon Jan 11 20:35:28 1999
  Modified:     Tue Jan 12 01:22:37 1999 (Erwin Dieterich) bamse@gmx.de
  Language:     C
  Package:      nameread
  Status:       Experimental (Do Not Distribute)
 
  (C) Copyright 1999, Erwin Dieterich, all rights reserved.
 
                                                                               
*/

/*   Main program to compare the performance of nameread with FreeType*/

#include <stdio.h>
#include <dirent.h>
#include <unistd.h>
#include <freetype.h>
#include "ftxerr18.h"
#include <time.h>

int nameread(char *filename, int *length);

/* Look for ttf-files */
int ttfselect(const struct dirent * d_entry)
{
  int   n;

  n = strlen(d_entry->d_name);
  if (n < 5)      /* Endung ".ttf" + 1 Buchstabe = 5 */
    return (0);
    
  /* Hier also > 4 */
  if ( strcmp(&d_entry->d_name[n-4],".ttf") && 
       strcmp(&d_entry->d_name[n-4],".TTF") && 
       strcmp(&d_entry->d_name[n-4],".txx"))    // ".txx" is for SoftMaker CDs
    return (0);  /* entry->d_name endet _nicht_ auf .ttf */
      
  return (1);    /* entry->d_name endet auf .ttf */

}


int main()
{
  
  char filename[256], stringf[256];
  int  i, l, ll, err, length;
  
  struct dirent **namelist;
  int   num_files, num_names;
  char  ttfdir[PATH_MAX];  /* is this safe ?? */

  TT_Engine  FontManEngine;
  TT_Face    tt_face;
  TT_Error   tt_error;
  TT_UShort  lu, pID, eID, lID, nID, lengthu;
  TT_String  *pstr;

  time_t time0, time1, time2, time3, time4;


  /* Look in the current directory or ttfdir for ".ttf"-files */
  getcwd(ttfdir,PATH_MAX);
  num_files = scandir(ttfdir, &namelist, ttfselect, alphasort);
  if ( num_files == 0 ) 
    {
      printf ("\nNo TrueType Files in this directory (%s).\n\n",ttfdir);
      exit(0);
    }
  printf("\n%i TTF-Files in Directory %s \n\n", num_files, ttfdir);

  printf("\n");
  printf("Nameread 1\n");
  printf("\n");

  time0 = time(NULL);

  for (i=0; i<num_files; i++)
    {
      strcpy (filename, namelist[i]->d_name );
      strcpy (stringf, filename);

      length=256;
      err = nameread(stringf, &length);
      if (err != 0)
	{
	  printf("Error in nameread: %s\n", filename);
	  if (err==-1)
	    perror(NULL);
	  printf("Error: %i\n", err);
	}

      /* Ugly conversion Unicode -> ASCII */
      for ( l=0; l<length/2; l++)
	stringf[l] = stringf[2*l+1];
      
      stringf[length/2] = '\0';
      
      /* printf ("%s: %s \n", filename, stringf); */
    }

  printf("\n");
  printf("Nameread 2\n");
  printf("\n");

  time1 = time(NULL);

  for (i=0; i<num_files; i++)
    {
      strcpy (filename, namelist[i]->d_name );
      strcpy (stringf, filename);

      length=256;
      err = nameread(stringf, &length);
      if (err != 0)
	{
	  printf("Error in nameread: %s\n", filename);
	  if (err==-1)
	    perror(NULL);
	  printf("Error: %i\n", err);
	}

      /* Ugly conversion Unicode -> ASCII */
      for ( l=0; l<length/2; l++)
	stringf[l] = stringf[2*l+1];
      
      stringf[length/2] = '\0';
      
      /* printf ("%s: %s \n", filename, stringf); */
    }

  printf("\n");
  printf("FreeType\n");
  printf("\n");
  time2 = time(NULL);

  /* FreeType */
  if ((tt_error = TT_Init_FreeType(&FontManEngine)))
    {
      printf( "ERROR: While initializing engine\n" );
      exit(-1);
    }

  for (i=0; i<num_files; i++)
    {
      strcpy (filename, namelist[i]->d_name );

      tt_error = TT_Open_Face( FontManEngine, filename, &tt_face );
      if (tt_error)
	exit (tt_error);

      num_names = TT_Get_Name_Count ( tt_face );
      
      lu=0;
      while ( lu<num_names )
	{

	  tt_error = TT_Get_Name_ID ( tt_face, lu, &pID, &eID, &lID, &nID );
	  if ( tt_error )
	      return (tt_error);

	  if ( pID == 3 && lID==1033 && nID==4 )
	    {
	      lengthu=256;
	      tt_error = TT_Get_Name_String ( tt_face, lu, &pstr, &lengthu );
	      if ( tt_error )
		{
		  printf("Error: %s\n", TT_ErrToString18( tt_error ));
		  return (tt_error);
		}

	      /* Ugly conversion Unicode -> ASCII */
	      for ( ll=0; ll<lengthu/2; ll++)
		stringf[ll] = pstr[2*ll+1];
	      
	      stringf[lengthu/2] = '\0';
	      
	      /* printf ("%s: %s \n", filename, stringf); */

	      lu = num_names; /*get out of loop */
	    }

	  lu+=1;
	}

      tt_error = TT_Close_Face( tt_face );
    }

  printf("\n");
  printf("FreeType 2\n");
  printf("\n");
  time3 = time(NULL);

  /* FreeType */
  if ((tt_error = TT_Init_FreeType(&FontManEngine)))
    {
      printf( "ERROR: While initializing engine\n" );
      exit(-1);
    }

  for (i=0; i<num_files; i++)
    {
      strcpy (filename, namelist[i]->d_name );

      tt_error = TT_Open_Face( FontManEngine, filename, &tt_face );
      if (tt_error)
	exit (tt_error);

      num_names = TT_Get_Name_Count ( tt_face );
      
      lu=0;
      while ( lu<num_names )
	{

	  tt_error = TT_Get_Name_ID ( tt_face, lu, &pID, &eID, &lID, &nID );
	  if ( tt_error )
	      return (tt_error);

	  if ( pID == 3 && lID==1033 && nID==4 )
	    {
	      lengthu=256;
	      tt_error = TT_Get_Name_String ( tt_face, lu, &pstr, &lengthu );
	      if ( tt_error )
		{
		  printf("Error: %s\n", TT_ErrToString18( tt_error ));
		  return (tt_error);
		}

	      /* Ugly conversion Unicode -> ASCII */
	      for ( ll=0; ll<lengthu/2; ll++)
		stringf[ll] = pstr[2*ll+1];
	      
	      stringf[lengthu/2] = '\0';
	      
	      /* printf ("%s: %s \n", filename, stringf); */

	      lu = num_names; /*get out of loop */
	    }

	  lu+=1;
	}

      tt_error = TT_Close_Face( tt_face );
    }


  printf("\n");

  time4 = time(NULL);

  printf ("Nameread 1: %f\n", difftime(time1, time0));
  printf ("Nameread 2: %f\n", difftime(time2, time1));
  printf ("FreeType 1: %f\n", difftime(time3, time2));
  printf ("FreeType 2: %f\n", difftime(time4, time3));

  printf("\n");
  printf("\n");
  exit(0);

}

