/* -*-C++-*-
                                                                               
 
  File:         nameread.cc
  RCS:          $Header: $
  Description:  
  Author:       Erwin Dieterich
  Created:      Sat Dec 26 23:11:22 1998
  Modified:     Mon Jan 11 22:27:21 1999 (Erwin Dieterich) bamse@gmx.de
  Language:     C++
  Package:      nameread
  Status:       Experimental (Do Not Distribute)
 
  (C) Copyright 1998-1999, Erwin Dieterich, all rights reserved.
 
                                                                               
*/

//#include <fstream> // Newer C++ standard? 
#include <stream.h>
#include <stdlib.h>
#include <string.h>

// is this always correct? What about Alphas and Crays?
// assume that unsigned char  == 8 bit 
// assume that unsigned short == 16 bit 
// assume that unsigned int   == 32 bit 
typedef unsigned char BYTE;
typedef unsigned short USHORT;
typedef unsigned int ULONG;

inline BYTE ReadBYTE(istream& font)
{
    BYTE c;
    font.get(c);
    return c;
}

USHORT ReadUSHORT(istream& font)
{
    USHORT us;
    us = ReadBYTE(font) << 8;
    us |= ReadBYTE(font);
    
    return us;
}

ULONG ReadULONG(istream& font)
{
    ULONG ul;
    ul = ReadBYTE(font) << 24;
    ul |= ReadBYTE(font) << 16;
    ul |= ReadBYTE(font) << 8;
    ul |= ReadBYTE(font);

    return ul;
}
//
//  Input:  filename: TrueType file
//          flength:  max. length of filename
//
//  Output: filename: TrueType name
//          flength:  length of TrueType name
//
int nameread(char* filename, int* flength)
{

  filebuf ttffile;
  char c1, c2, c3, c4;
  ULONG checksum, offset, length;
  USHORT format, num_names, num_tables, noffset;
  USHORT pID, eID, lID, nID, slength, soffset;
  int i,l;

  if (ttffile.open(filename, input) == 0)
      return(-1);

  istream font(&ttffile);

  // We are (right now) not interested in the version number
  font.seekg(4); 

  num_tables = ReadUSHORT(font);
  if (num_tables > 50) /* Better test available? */
    {
      ttffile.close();
      return(-5);
    }

  // We are (right now) not interested in the rest of the table dir
  font.seekg(12); 

  // Now the tables
  i=0;
  offset=0;
  c1 = c2 = c3 = c4 = ' ';
  while ( offset==0 && i<num_tables )
    {
      
      //table tags
      c1 = (char) ReadBYTE(font);
      c2 = (char) ReadBYTE(font);
      c3 = (char) ReadBYTE(font);
      c4 = (char) ReadBYTE(font);

      // In the current technical specification of the TrueType format
      // the only table beginning with 'n' is the name table,
      // but we are paranoid: 
      if ( c1 == 'n' && c2 == 'a' && c3 == 'm' && c4 == 'e' )
	{
	  checksum = ReadULONG(font);
	  offset   = ReadULONG(font);
	  length   = ReadULONG(font);
	}

      // replace with file positioning (seek) ?!
      ReadULONG(font);
      ReadULONG(font);
      ReadULONG(font);

      i++;
    }

  if ( offset==0 )
    {
      ttffile.close();
      return(-3);
    }

  font.seekg(offset); // is this still o.k. for new c++ standard ??


  // Beginning of the name table
  format    = ReadUSHORT(font);
  num_names = ReadUSHORT(font);
  noffset   = ReadUSHORT(font);

  // the name table should be sorted by platform, encoding, language, name,
  // but can we be sure?

  // We are looking for:   Microsoft platform   (pID=3)
  //                       encoding unimportant (1: text font, 0: symbol font)
  //                       American English     (lID=1033)
  //                       Full Font Name       (nID=4)

  i=0;
  pID = eID = lID = nID = 0;
  while ( i<num_names)
    {
      font.seekg(offset+6+i*12); // is this still o.k. for new c++ ??
      pID = ReadUSHORT(font);
      if (pID==3)
	{
	  eID = ReadUSHORT(font);
	  lID = ReadUSHORT(font);
	  nID = ReadUSHORT(font);
	  if ( lID==1033 && nID==4 )
	    {
	      slength = ReadUSHORT(font);
	      soffset = ReadUSHORT(font);
	      font.seekg(offset+noffset+soffset);

	      // If the length of the TrueType name is bigger than
	      // what can be stored in filename, shorten it.
	      if ( slength<*flength )
		*flength = slength;

	      for ( l=0; l<*flength; l++)
		  filename[l] = ReadBYTE(font);

	      ttffile.close();
	      return(0);
	    }

	}

      i++;
    }
     
  ttffile.close();
  return(-2);


}

