/* -*-C-*-
                                                                               
 
  File:         nameread.cc
  RCS:          $Header: $
  Description:  
  Author:       Erwin Dieterich
  Created:      Mon Jan 11 21:57:01 1999
  Modified:     Mon Jan 11 23:51:35 1999 (Erwin Dieterich) bamse@gmx.de
  Language:     C
  Package:      nameread
  Status:       Experimental (Do Not Distribute)
 
  (C) Copyright 1999, Erwin Dieterich, all rights reserved.
 
                                                                               
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* is this always correct? What about Alphas and Crays? */
/* assume that unsigned char  == 8 bit */
/* assume that unsigned short == 16 bit */
/* assume that unsigned int   == 32 bit */
typedef unsigned char BYTE;
typedef unsigned short USHORT;
typedef unsigned int ULONG;

inline BYTE ReadBYTE( FILE *font)
{
    BYTE c;
    c = fgetc(font);
    return c;
}

USHORT ReadUSHORT( FILE *font)
{
    USHORT us;
    us = ReadBYTE(font) << 8;
    us |= ReadBYTE(font);
    
    return us;
}

ULONG ReadULONG( FILE *font)
{
    ULONG ul;
    ul = ReadBYTE(font) << 24;
    ul |= ReadBYTE(font) << 16;
    ul |= ReadBYTE(font) << 8;
    ul |= ReadBYTE(font);

    return ul;
}
/*
    Input: filename: TrueType file
           flength:  max. length of filename

   Output: filename: TrueType name
           flength:  length of TrueType name
*/
int nameread(char* filename, int* flength)
{

  FILE *ttffile;
  char c1, c2, c3, c4;
  ULONG checksum, offset, length;
  USHORT format, num_names, num_tables, noffset;
  USHORT pID, eID, lID, nID, slength, soffset;
  int i,l;

  if ( (ttffile=fopen(filename, "r")) == 0)
      return(-1);

  /* We are (right now) not interested in the version number */
  fseek(ttffile, 4, SEEK_SET); 

  num_tables = ReadUSHORT(ttffile);
  if (num_tables > 50) /* Better test available? */
    {
      fclose(ttffile);
      return(-5);
    }

  /* We are (right now) not interested in the rest of the table dir */
  fseek(ttffile, 12, SEEK_SET); 

  /* Now the tables */
  i=0;
  offset=0;
  c1 = c2 = c3 = c4 = ' ';
  while ( offset==0 && i<num_tables )
    {
      
      /* table tags */
      c1 = (char) ReadBYTE(ttffile);
      c2 = (char) ReadBYTE(ttffile);
      c3 = (char) ReadBYTE(ttffile);
      c4 = (char) ReadBYTE(ttffile);

      /* In the current technical specification of the TrueType format  */
      /* the only table beginning with 'n' is the name table,           */
      /* but we are paranoid:                                           */
      if ( c1 == 'n' && c2 == 'a' && c3 == 'm' && c4 == 'e' )
	{
	  checksum = ReadULONG(ttffile);
	  offset   = ReadULONG(ttffile);
	  length   = ReadULONG(ttffile);
	}

      /* replace with file positioning (fseek) ?! */
      ReadULONG(ttffile);
      ReadULONG(ttffile);
      ReadULONG(ttffile);

      i++;
    }

  if ( offset==0 )
    {
      fclose(ttffile);
      return(-3);
    }

  fseek(ttffile, offset, SEEK_SET); 


  /* Beginning of the name table */
  format    = ReadUSHORT(ttffile);
  num_names = ReadUSHORT(ttffile);
  noffset   = ReadUSHORT(ttffile);

  /*
  the name table should be sorted by platform, encoding, language, name,
  but can we be sure?

  We are looking for:   Microsoft platform   (pID=3)
                        encoding unimportant (1: text font, 0: symbol font)
                        American English     (lID=1033)
                        Full Font Name       (nID=4)
  */

  i=0;
  pID = eID = lID = nID = 0;
  while ( i<num_names)
    {
      fseek(ttffile, offset+6+i*12, SEEK_SET); 
      pID = ReadUSHORT(ttffile);
      if (pID==3)
	{
	  eID = ReadUSHORT(ttffile);
	  lID = ReadUSHORT(ttffile);
	  nID = ReadUSHORT(ttffile);
	  if ( lID==1033 && nID==4 )
	    {
	      slength = ReadUSHORT(ttffile);
	      soffset = ReadUSHORT(ttffile);
	      fseek(ttffile, offset+noffset+soffset, SEEK_SET);

	      /* If the length of the TrueType name is bigger than  */
	      /* what can be stored in filename, shorten it.        */
	      if ( slength<*flength )
		*flength = slength;

	      for ( l=0; l<*flength; l++)
		  filename[l] = ReadBYTE(ttffile);

	      if (fclose(ttffile)==EOF)
		perror(NULL);
	      return(0);
	    }

	}

      i++;
    }
     
  fclose(ttffile);
  return(-2);


}

